@@ -1,10 +1,14 @@
 """Traits belonging to camera devices."""
 
+from __future__ import annotations
+
 import datetime
 import urllib.parse as urlparse
+from typing import Any, List, Mapping, Optional, cast
 
 from .event import CameraMotionEvent, CameraPersonEvent, CameraSoundEvent, EventTrait
 from .traits import TRAIT_MAP, Command
+from .typing import cast_assert, cast_optional
 
 MAX_IMAGE_RESOLUTION = "maxImageResolution"
 MAX_VIDEO_RESOLUTION = "maxVideoResolution"
@@ -25,8 +29,8 @@ EXPIRES_AT = "expiresAt"
 class Resolution:
     """Maximum Resolution of an image or stream."""
 
-    width = None
-    height = None
+    width: Optional[int] = None
+    height: Optional[int] = None
 
 
 @TRAIT_MAP.register()
@@ -35,7 +39,7 @@ class CameraImageTrait:
 
     NAME = "sdm.devices.traits.CameraImage"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize CameraImageTrait."""
         self._data = data
         self._cmd = cmd
@@ -52,7 +56,7 @@ class CameraImageTrait:
 class RtspStream:
     """Provides access an RTSP live stream URL."""
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize RstpStream."""
         self._data = data
         self._cmd = cmd
@@ -60,17 +64,21 @@ class RtspStream:
     @property
     def rtsp_stream_url(self) -> str:
         """RTSP live stream URL."""
-        return self._data[STREAM_URLS][RTSP_URL]
+        rtsp_stream_url = self._data[STREAM_URLS][RTSP_URL]
+        assert isinstance(rtsp_stream_url, str)
+        return rtsp_stream_url
 
     @property
     def stream_token(self) -> str:
         """Token to use to access an RTSP live stream."""
-        return self._data[STREAM_TOKEN]
+        stream_token = self._data[STREAM_TOKEN]
+        assert isinstance(stream_token, str)
+        return stream_token
 
     @property
     def stream_extension_token(self) -> str:
         """Token to use to access an RTSP live stream."""
-        return self._data[STREAM_EXTENSION_TOKEN]
+        return cast_assert(str, self._data[STREAM_EXTENSION_TOKEN])
 
     @property
     def expires_at(self) -> datetime.datetime:
@@ -78,7 +86,7 @@ class RtspStream:
         expires_at = self._data[EXPIRES_AT]
         return datetime.datetime.fromisoformat(expires_at.replace("Z", "+00:00"))
 
-    async def extend_rtsp_stream(self):
+    async def extend_rtsp_stream(self) -> RtspStream:
         """Request a new RTSP live stream URL access token."""
         data = {
             "command": "sdm.devices.commands.CameraLiveStream.ExtendRtspStream",
@@ -96,7 +104,7 @@ class RtspStream:
         results[STREAM_URLS][RTSP_URL] = url
         return RtspStream(results, self._cmd)
 
-    async def stop_rtsp_stream(self):
+    async def stop_rtsp_stream(self) -> None:
         """Invalidates a valid RTSP access token and stops the RTSP live stream."""
         data = {
             "command": "sdm.devices.commands.CameraLiveStream.StopRtspStream",
@@ -111,7 +119,7 @@ class CameraLiveStreamTrait:
 
     NAME = "sdm.devices.traits.CameraLiveStream"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize CameraLiveStreamTrait."""
         self._data = data
         self._cmd = cmd
@@ -125,14 +133,15 @@ class CameraLiveStreamTrait:
         return res
 
     @property
-    def video_codecs(self) -> list:
+    def video_codecs(self) -> List[str]:
         """Video codecs supported for the live stream."""
-        return self._data[VIDEO_CODECS]
+        assert isinstance(self._data[VIDEO_CODECS], list)
+        return cast(List[str], self._data[VIDEO_CODECS])
 
     @property
-    def audio_codecs(self) -> list:
+    def audio_codecs(self) -> List[str]:
         """Audio codecs supported for the live stream."""
-        return self._data[AUDIO_CODECS]
+        return cast(List[str], self._data[AUDIO_CODECS])
 
     async def generate_rtsp_stream(self) -> RtspStream:
         """Request a token to access an RTSP live stream URL."""
@@ -158,28 +167,31 @@ class EventImage:
     Authorization: Basic <token>
     """
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize the EventImage."""
         self._data = data
         self._cmd = cmd
 
     @property
-    def url(self) -> str:
+    def url(self) -> Optional[str]:
         """URL to download the camera image from."""
-        return self._data[URL]
+        return cast_optional(str, self._data.get(URL))
 
     @property
-    def token(self) -> str:
+    def token(self) -> Optional[str]:
         """Token to use in the HTTP Authorization header when downloading."""
-        return self._data[TOKEN]
+        return cast_optional(str, self._data.get(TOKEN))
 
-    async def contents(self, width=None, height=None) -> bytes:
+    async def contents(
+        self, width: Optional[int] = None, height: Optional[int] = None
+    ) -> bytes:
         """Download the image bytes."""
         if width:
             fetch_url = f"{self.url}?width={width}"
         elif height:
             fetch_url = f"{self.url}?width={height}"
         else:
+            assert self.url
             fetch_url = self.url
         return await self._cmd.fetch_image(fetch_url, basic_auth=self.token)
 
@@ -190,7 +202,7 @@ class CameraEventImageTrait:
 
     NAME = "sdm.devices.traits.CameraEventImage"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize CameraEventImageTrait."""
         self._data = data
         self._cmd = cmd
@@ -216,18 +228,19 @@ class CameraMotionTrait(EventTrait):
     NAME = "sdm.devices.traits.CameraMotion"
     EVENT_NAME = CameraMotionEvent.NAME
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize CameraMotionTrait."""
         super().__init__()
         self._data = data
         self._cmd = cmd
         self._event_image = CameraEventImageTrait({}, cmd)
 
-    async def generate_active_event_image(self) -> EventImage:
+    async def generate_active_event_image(self) -> Optional[EventImage]:
         """Provide a URL to download a camera image from the active event."""
         event = self.active_event
         if not event:
             return None
+        assert event.event_id
         return await self._event_image.generate_image(event.event_id)
 
 
@@ -238,18 +251,19 @@ class CameraPersonTrait(EventTrait):
     NAME = "sdm.devices.traits.CameraPerson"
     EVENT_NAME = CameraPersonEvent.NAME
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize CameraPersonTrait."""
         super().__init__()
         self._data = data
         self._cmd = cmd
         self._event_image = CameraEventImageTrait({}, cmd)
 
-    async def generate_active_event_image(self) -> EventImage:
+    async def generate_active_event_image(self) -> Optional[EventImage]:
         """Provide a URL to download a camera image from the active event."""
         event = self.active_event
         if not event:
             return None
+        assert event.event_id
         return await self._event_image.generate_image(event.event_id)
 
 
@@ -260,16 +274,17 @@ class CameraSoundTrait(EventTrait):
     NAME = "sdm.devices.traits.CameraSound"
     EVENT_NAME = CameraSoundEvent.NAME
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize CameraSoundTrait."""
         super().__init__()
         self._data = data
         self._cmd = cmd
         self._event_image = CameraEventImageTrait({}, cmd)
 
-    async def generate_active_event_image(self) -> EventImage:
+    async def generate_active_event_image(self) -> Optional[EventImage]:
         """Provide a URL to download a camera image from the active event."""
         event = self.active_event
         if not event:
             return None
+        assert event.event_id
         return await self._event_image.generate_image(event.event_id)
