@@ -11,6 +11,7 @@ from aiohttp.client_exceptions import ClientError
 from google.api_core.exceptions import GoogleAPIError, NotFound, Unauthenticated
 from google.auth.exceptions import RefreshError
 from google.auth.transport.requests import Request
+from google.oauth2.credentials import Credentials
 from google.cloud import pubsub_v1
 
 from .auth import AbstractAuth
@@ -41,7 +42,13 @@ class AbstractSubscriberFactory(ABC):
 
     @abstractmethod
     async def async_new_subscriber(
-        self, creds, subscription_name, async_callback
+        self,
+        creds: Credentials,
+        subscription_name: str,
+        loop: asyncio.AbstractEventLoop,
+        async_callback: Callable[
+            [pubsub_v1.subscriber.message.Message], Awaitable[None]
+        ],
     ) -> pubsub_v1.subscriber.futures.StreamingPullFuture:
         """Create a new event subscriber."""
 
@@ -50,14 +57,23 @@ class DefaultSubscriberFactory(AbstractSubscriberFactory):
     """Default implementation that creates Google Pubsub subscriber."""
 
     async def async_new_subscriber(
-        self, creds, subscription_name, loop, async_callback
+        self,
+        creds: Credentials,
+        subscription_name: str,
+        loop: asyncio.AbstractEventLoop,
+        async_callback: Callable[
+            [pubsub_v1.subscriber.message.Message], Awaitable[None]
+        ],
     ) -> pubsub_v1.subscriber.futures.StreamingPullFuture:
         """Create a new subscriber with a blocking to async bridge."""
 
-        def callback_wrapper(message: pubsub_v1.subscriber.message.Message):
+        def callback_wrapper(message: pubsub_v1.subscriber.message.Message) -> None:
             if loop.is_closed():
                 return
-            future = asyncio.run_coroutine_threadsafe(async_callback(message), loop)
+            assert async_callback
+            future: concurrent.futures.Future = asyncio.run_coroutine_threadsafe(
+                async_callback(message), loop
+            )
             future.result()
 
         with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
@@ -69,7 +85,12 @@ class DefaultSubscriberFactory(AbstractSubscriberFactory):
                 callback_wrapper,
             )
 
-    def _new_subscriber(self, creds, subscription_name, callback_wrapper):
+    def _new_subscriber(
+        self,
+        creds: Credentials,
+        subscription_name: str,
+        callback_wrapper: Callable[[pubsub_v1.subscriber.message.Message], None],
+    ) -> pubsub_v1.subscriber.futures.StreamingPullFuture:
         """Issue a command to verify subscriber creds are correct."""
 
         try:
@@ -113,24 +134,27 @@ class GoogleNestSubscriber:
         self._subscriber_id = subscriber_id
         self._api = GoogleNestAPI(auth, project_id)
         self._loop = loop or asyncio.get_event_loop()
-        self._device_manager_task = None
+        self._device_manager_task: Optional[asyncio.Task[DeviceManager]] = None
         self._subscriber_factory = subscriber_factory
-        self._subscriber_future = None
-        self._callback = None
+        self._subscriber_future: Optional[
+            pubsub_v1.subscriber.futures.StreamingPullFuture
+        ] = None
+        self._callback: Optional[Callable[[EventMessage], Awaitable[None]]] = None
         self._healthy = True
         self._watchdog_check_interval_seconds = watchdog_check_interval_seconds
         self._watchdog_restart_delay_min_seconds = watchdog_restart_delay_min_seconds
         self._watchdog_restart_delay_seconds = watchdog_restart_delay_min_seconds
+        self._watchdog_task: Optional[asyncio.Task] = None
         if self._watchdog_check_interval_seconds > 0:
             self._watchdog_task = asyncio.create_task(self._watchdog())
-        else:
-            self._watchdog_task = None
 
-    def set_update_callback(self, target: Callable[[EventMessage], Awaitable[None]]):
+    def set_update_callback(
+        self, target: Callable[[EventMessage], Awaitable[None]]
+    ) -> None:
         """Register a callback invoked when new messages are received."""
         self._callback = target
 
-    async def start_async(self):
+    async def start_async(self) -> None:
         """Start the subscriber."""
         if not EXPECTED_SUBSCRIBER_REGEXP.match(self._subscriber_id):
             raise ConfigurationException(
@@ -169,7 +193,7 @@ class GoogleNestSubscriber:
         if self._subscriber_future:
             self._subscriber_future.add_done_callback(self._done_callback)
 
-    async def _watchdog(self):
+    async def _watchdog(self) -> None:
         """Background task that watches the subscriber and restarts it."""
         _LOGGER.debug("Starting background watchdog thread")
         while True:
@@ -187,11 +211,12 @@ class GoogleNestSubscriber:
                 await self.start_async()
             await asyncio.sleep(self._watchdog_check_interval_seconds)
 
-    def wait(self):
+    def wait(self) -> None:
         """Block on the subscriber."""
+        assert self._subscriber_future
         self._subscriber_future.result()
 
-    def stop_async(self):
+    def stop_async(self) -> None:
         """Tell the subscriber to start shutting down."""
         if self._device_manager_task:
             self._device_manager_task.cancel()
@@ -206,9 +231,10 @@ class GoogleNestSubscriber:
             self._device_manager_task = asyncio.create_task(
                 self._async_create_device_manager()
             )
+        assert self._device_manager_task
         return await self._device_manager_task
 
-    async def _async_create_device_manager(self):
+    async def _async_create_device_manager(self) -> DeviceManager:
         """Create a DeviceManager, populated with initial state."""
         device_manager = DeviceManager()
         structures = await self._api.async_get_structures()
@@ -220,7 +246,9 @@ class GoogleNestSubscriber:
             device_manager.add_device(device)
         return device_manager
 
-    def _done_callback(self, future):
+    def _done_callback(
+        self, future: pubsub_v1.subscriber.futures.StreamingPullFuture
+    ) -> None:
         """Teardown subscriber."""
         if future.exception():
             ex = future.exception()
@@ -230,7 +258,7 @@ class GoogleNestSubscriber:
 
     async def _async_message_callback(
         self, message: pubsub_v1.subscriber.message.Message
-    ):
+    ) -> None:
         """Handle a received message."""
         payload = json.loads(bytes.decode(message.data))
         event = EventMessage(payload, self._auth)
