@@ -1,11 +1,19 @@
 """Traits for thermostats."""
-from .traits import TRAIT_MAP, Command
 
-STATUS = "status"
-AVAILABLE_MODES = "availableModes"
-MODE = "mode"
-HEAT_CELSIUS = "heatCelsius"
-COOL_CELSIUS = "coolCelsius"
+from __future__ import annotations
+
+from typing import Any, Final, List, Mapping, cast
+
+import aiohttp
+
+from .traits import TRAIT_MAP, Command
+from .typing import cast_assert
+
+STATUS: Final = "status"
+AVAILABLE_MODES: Final = "availableModes"
+MODE: Final = "mode"
+HEAT_CELSIUS: Final = "heatCelsius"
+COOL_CELSIUS: Final = "coolCelsius"
 
 
 @TRAIT_MAP.register()
@@ -14,22 +22,22 @@ class ThermostatEcoTrait:
 
     NAME = "sdm.devices.traits.ThermostatEco"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize ThermostatEcoTrait."""
         self._data = data
         self._cmd = cmd
 
     @property
-    def available_modes(self) -> list:
+    def available_modes(self) -> List[str]:
         """List of supported Eco modes."""
-        return self._data[AVAILABLE_MODES]
+        return cast(List[str], self._data[AVAILABLE_MODES])
 
     @property
     def mode(self) -> str:
         """Eco mode of the thermostat."""
-        return self._data[MODE]
+        return cast_assert(str, self._data[MODE])
 
-    async def set_mode(self, mode):
+    async def set_mode(self, mode: str) -> aiohttp.ClientResponse:
         """Change the thermostat Eco mode."""
         data = {
             "command": "sdm.devices.commands.ThermostatEco.SetMode",
@@ -40,12 +48,12 @@ class ThermostatEcoTrait:
     @property
     def heat_celsius(self) -> float:
         """Lowest temperature where Eco mode begins heating."""
-        return self._data[HEAT_CELSIUS]
+        return cast_assert(float, self._data[HEAT_CELSIUS])
 
     @property
     def cool_celsius(self) -> float:
         """Highest cooling temperature where Eco mode begins cooling."""
-        return self._data[COOL_CELSIUS]
+        return cast_assert(float, self._data[COOL_CELSIUS])
 
 
 @TRAIT_MAP.register()
@@ -54,14 +62,14 @@ class ThermostatHvacTrait:
 
     NAME = "sdm.devices.traits.ThermostatHvac"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize ThermostatHvacTrait."""
         self._data = data
 
     @property
-    def status(self) -> list:
+    def status(self) -> List[str]:
         """HVAC status of the thermostat."""
-        return self._data[STATUS]
+        return cast(List[str], self._data[STATUS])
 
 
 @TRAIT_MAP.register()
@@ -70,22 +78,22 @@ class ThermostatModeTrait:
 
     NAME = "sdm.devices.traits.ThermostatMode"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize ThermostatModeTrait."""
         self._data = data
         self._cmd = cmd
 
     @property
-    def available_modes(self) -> list:
+    def available_modes(self) -> List[str]:
         """List of supported thermostat modes."""
-        return self._data[AVAILABLE_MODES]
+        return cast(List[str], self._data[AVAILABLE_MODES])
 
     @property
     def mode(self) -> str:
         """Mode of the thermostat."""
-        return self._data[MODE]
+        return cast_assert(str, self._data[MODE])
 
-    async def set_mode(self, mode):
+    async def set_mode(self, mode: str) -> aiohttp.ClientResponse:
         """Change the thermostat Eco mode."""
         data = {
             "command": "sdm.devices.commands.ThermostatMode.SetMode",
@@ -100,7 +108,7 @@ class ThermostatTemperatureSetpointTrait:
 
     NAME = "sdm.devices.traits.ThermostatTemperatureSetpoint"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize ThermostatTemperatureSetpointTrait."""
         self._data = data
         self._cmd = cmd
@@ -108,14 +116,14 @@ class ThermostatTemperatureSetpointTrait:
     @property
     def heat_celsius(self) -> float:
         """Lowest temperature where Eco mode begins heating."""
-        return self._data[HEAT_CELSIUS]
+        return cast_assert(float, self._data[HEAT_CELSIUS])
 
     @property
-    def cool_celsius(self) -> list:
+    def cool_celsius(self) -> float:
         """Highest cooling temperature where Eco mode begins cooling."""
-        return self._data[COOL_CELSIUS]
+        return cast_assert(float, self._data[COOL_CELSIUS])
 
-    async def set_heat(self, heat: float):
+    async def set_heat(self, heat: float) -> aiohttp.ClientResponse:
         """Change the thermostat Eco mode."""
         data = {
             "command": "sdm.devices.commands.ThermostatTemperatureSetpoint.SetHeat",
@@ -123,7 +131,7 @@ class ThermostatTemperatureSetpointTrait:
         }
         return await self._cmd.execute(data)
 
-    async def set_cool(self, cool: float):
+    async def set_cool(self, cool: float) -> aiohttp.ClientResponse:
         """Change the thermostat Eco mode."""
         data = {
             "command": "sdm.devices.commands.ThermostatTemperatureSetpoint.SetCool",
@@ -131,7 +139,7 @@ class ThermostatTemperatureSetpointTrait:
         }
         return await self._cmd.execute(data)
 
-    async def set_range(self, heat: float, cool: float):
+    async def set_range(self, heat: float, cool: float) -> aiohttp.ClientResponse:
         """Change the thermostat Eco mode."""
         data = {
             "command": "sdm.devices.commands.ThermostatTemperatureSetpoint.SetRange",
